/*
 * LyTreemapBehaviorInterface.java
 * Copyright (c) 2008, Lyria.
 * All rights reserved.
 */

package leon.app.behaviorinterface;

import leon.control.*;
import leon.data.*;
import leon.info.*;
import leon.misc.*;


/**
 * This interface defines the methods to implement in order to modify the behavior of treemap view
 * controller.
 *
 * @author  Lyria S.A.
 * @version $Id: LyTreemapBehaviorInterface.java,v 6.5 2007/12/05 09:01:03 jls Exp $
 * @see     leon.control.LyTreemapController
 */
public interface LyTreemapBehaviorInterface extends LyActionBehaviorInterface
{
	String __VERSION = "$Id: LyTreemapBehaviorInterface.java,v 6.5 2007/12/05 09:01:03 jls Exp $";

	/**
	 * Gets the list of fields that may be used by the treemap hierarchy.
	 *
	 * @param  controller the treemap controller
	 * @return the list of fields that may be used by the treemap hierarchy
	 */
	LyInfoList<LyFieldInfo> getHierarchyFieldList(LyTreemapController controller);

	/**
	 * Gets the initial list of fields used to compute the treemap hierarchy, as defined in the
	 * metamodel.
	 *
	 * @param  controller the treemap controller
	 * @return the list of fields used to compute the treemap hierarchy.
	 */
	LyInfoList<LyFieldInfo> getHierarchyFields(LyTreemapController controller);

	/**
	 * Gets the initial number of displayed levels of the treemap hierarchy, as defined in the
	 * metamodel.
	 *
	 * @param  controller the treemap controller
	 * @return the number of displayed levels of the treemap hierarchy.
	 */
	int getHierarchyLevels(LyTreemapController controller);

	/**
	 * Gets the list of fields that may be used to compute the size of treemap nodes.
	 *
	 * @param  controller the treemap controller
	 * @return the list of fields that may be used to compute the size of treemap nodes.
	 */
	LyInfoList<LyFieldInfo> getSizeFieldList(LyTreemapController controller);

	/**
	 * Gets the initial field used to compute the size of treemap nodes, as defined in the
	 * metamodel.
	 *
	 * @param  controller the treemap controller
	 * @return the field used to compute the size of treemap nodes.
	 */
	LyFieldInfo getSizeField(LyTreemapController controller);

	/**
	 * Gets the list of fields that may be used to compute the color of treemap nodes.
	 *
	 * @param  controller the treemap controller
	 * @return the list of fields that may be used to compute the color of treemap nodes.
	 */
	LyInfoList<LyFieldInfo> getColorFieldList(LyTreemapController controller);

	/**
	 * Gets the initial field used to compute the color of treemap nodes, as defined in the
	 * metamodel.
	 *
	 * @param  controller the treemap controller
	 * @return the field used to compute the color of treemap nodes.
	 */
	LyFieldInfo getColorField(LyTreemapController controller);

	/**
	 * Gets the initial color used for the minimum value of treemap nodes.
	 *
	 * @param  controller the treemap controller
	 * @return the color used for the minimum value of treemap nodes.
	 */
	String getMinColor(LyTreemapController controller);

	/**
	 * Gets the initial color used for the maximum value of treemap nodes.
	 *
	 * @param  controller the treemap controller
	 * @return the color used for the maximum value of treemap nodes.
	 */
	String getMaxColor(LyTreemapController controller);

	/**
	 * Gets the list of supported aggregation algorithms.
	 *
	 * @param  controller the treemap controller
	 * @return Array of couple of strings that represents supported algorithms. Each couple of
	 *         strings is the algorithm code for the first string, and the displayable name of the
	 *         algorithm or a key to the dictionnary of this string for the second.
	 */
	String[][] getAggregationTypes(LyTreemapController controller);

	/**
	 * Gets the code of the initial algorithm used for aggregation by the treemap.
	 *
	 * @param  controller the treemap controller
	 * @return the code of the algorithm used for aggregation by the treemap.
	 */
	String getAggregationType(LyTreemapController controller);

	/**
	 * Gets the list of fields that may be used by the aggregation algorithms.
	 *
	 * @param  controller the treemap controller
	 * @return the list of fields that may be used by the aggregation algorithms.
	 */
	LyInfoList<LyFieldInfo> getAggregationFieldList(LyTreemapController controller);

	/**
	 * Gets the initial field used by current aggregation algorithm, as defined in the metamodel.
	 *
	 * @param  controller the treemap controller
	 * @return the field used by current aggregation algorithm.
	 */
	LyFieldInfo getAggregationField(LyTreemapController controller);

	/**
	 * Gets the displayable label of a map region from the given field info and the given object.
	 *
	 * @param  controller the treemap controller
	 * @param  fieldInfo  the field info corresponding to the map region. If the region represents a
	 *                    final object of the hierarchy, this parameter is null. Otherwise it
	 *                    contains the field info used at the level of the map region (should be one
	 *                    of the field infos used in the hierarchy).
	 * @param  object     the object describing the map region. If map region does not represent a
	 *                    final object of the hierarchy, the object is the first one contained in
	 *                    the map region.
	 * @return the displayable name for the given map region
	 */
	String getMapRegionLabel(LyTreemapController controller, LyFieldInfo fieldInfo,
		LyObject object);

	/**
	 * Gets the tooltip for a map region from the given parameters.
	 *
	 * @param  controller  the treemap controller
	 * @param  fieldInfo   the field info corresponding to the map region. If the region represents
	 *                     a final object of the hierarchy, this parameter is null. Otherwise it
	 *                     contains the field info used at the level of the map region (should be
	 *                     one of the field infos used in the hierarchy).
	 * @param  object      the object describing the map region. If map region does not represent a
	 *                     final object of the hierarchy, the object is the first one contained in
	 *                     the map region.
	 * @param  weightValue the value used to represent the weight of this region
	 * @param  colorValue  the value used to represent the color of this region
	 * @return the tooltip for the given map region
	 */
	String getMapRegionTooltip(LyTreemapController controller, LyFieldInfo fieldInfo,
		LyObject object, float weightValue, float colorValue);

	/**
	 * Gets a small image for the map region associated to the given object.
	 *
	 * @param  controller the treemap controller
	 * @param  fieldInfo  the field info corresponding to the map region. If the region represents a
	 *                    final object of the hierarchy, this parameter is null. Otherwise it
	 *                    contains the field info used at the level of the map region (should be one
	 *                    of the field infos used in the hierarchy).
	 * @param  object     the object describing the map region. If map region does not represent a
	 *                    final object of the hierarchy, the object is the first one contained in
	 *                    the map region.
	 * @return the small image for the given map region.
	 */
	String getMapRegionSmallImage(LyTreemapController controller, LyFieldInfo fieldInfo,
		LyObject object);

	/**
	 * Gets the tooltip for the small image of a map region.
	 *
	 * @param  controller the treemap controller
	 * @param  fieldInfo  the field info corresponding to the map region. If the region represents a
	 *                    final object of the hierarchy, this parameter is null. Otherwise it
	 *                    contains the field info used at the level of the map region (should be one
	 *                    of the field infos used in the hierarchy).
	 * @param  object     the object describing the map region. If map region does not represent a
	 *                    final object of the hierarchy, the object is the first one contained in
	 *                    the map region.
	 * @return the tooltip for the small image of the given map region
	 */
	String getMapRegionSmallImageTooltip(LyTreemapController controller,
		LyFieldInfo fieldInfo, LyObject object);

	/**
	 * Computes the value used to represent the size of the given object.
	 *
	 * @param  controller the treemap controller
	 * @param  sizeField  the field info used to compute sizes
	 * @param  object     the object used to compute size for a given map region
	 * @return the value used to represent the size for a given object.
	 */
	Number computeSizeValue(LyTreemapController controller, LyFieldInfo sizeField,
		LyObject object);

	/**
	 * Computes the value used to represent the color for a given list of objects.
	 *
	 * @param  controller  the treemap controller
	 * @param  colorField  the field info used to compute colors
	 * @param  objects     the list of objects used to compute color for a given map region
	 * @param  isHierarchy indicates if the color will be used for a hierarchy map region or a final
	 *                     map region
	 * @return the value used to represent the color for a given list of objects.
	 */
	Number computeColorValue(LyTreemapController controller, LyFieldInfo colorField,
		LyObjectList objects, boolean isHierarchy);

	/**
	 * Computes a numeric value for the given aggregation algorithm type and given numeric values.
	 *
	 * @param  controller      the treemap controller
	 * @param  aggregationType the code of the algorithm to be used.
	 * @param  values          the values that must be used
	 * @param  balanceValues   the values that must be used to balance the given values. Some
	 *                         algorithms do not use these values.
	 * @param  valuesNumber    indicate the number of values that must be taken into account
	 * @return a numeric value for a specific algorithm and given numeric values.
	 */
	float computeAggregationValue(LyTreemapController controller, String aggregationType,
		float[] values, float[] balanceValues, int valuesNumber);

	/**
	 * Computes a color from the given values and the two given colors.
	 *
	 * @param  controller the treemap controller
	 * @param  value      the value used to compute the color
	 * @param  minValue   the value corresponding to the minimum color
	 * @param  maxValue   the value corresponding to the maximum color
	 * @param  minColor   the minimum color
	 * @param  maxColor   the maximum color
	 * @return the color (as a color key or an RGB code) computed from the given value and min and
	 *         max colors.
	 */
	String computeColor(LyTreemapController controller, float value, float minValue,
		float maxValue, LyColor minColor, LyColor maxColor);

	/**
	 * Gets the treemap title from the digen array of labels. This title depends on the current
	 * position in the treemap hierarchy.
	 *
	 * @param  controller the treemap controller
	 * @param  labels     the list of the regions labels used to reach current position in the
	 *                    hierarchy
	 * @return the title of the treemap
	 */
	String getTreemapTitle(LyTreemapController controller, String[][] labels);

	/**
	 * This method is invoked when the view is ready to be shown. May be overriden to add extra
	 * components.
	 *
	 * @param controller the treemap controller
	 */
	void completeTreemap(LyTreemapController controller);

	/**
	 * Indicates if the current user configuration must be saved and restored.
	 *
	 * @param  controller the treemap controller
	 * @return true if the configuration has to be saved, false otherwise.
	 */
	boolean saveUserConfiguration(LyTreemapController controller);

	/**
	 * Indicates if mono selection must be used in treemap.
	 *
	 * @param  controller the treemap controller
	 * @return true if mono selection must be used, false if multi selection must be used
	 */
	boolean useMonoSelection(LyTreemapController controller);

	/**
	 * Indicates if the treemap component must be refreshed when a selection occurs.
	 *
	 * @param  controller the treemap controller
	 * @return true if treemap component must be refreshed when a selection occurs, false otherwise
	 */
	boolean refreshViewOnSelection(LyTreemapController controller);

	/**
	 * Frees all resources to close the treemap controller.
	 *
	 * @param controller the treemap controller
	 */
	void free(LyTreemapController controller);
}